# include <anime.h>

static void my_sprite_draw(mySprite *sprite);
static void my_anime_area_draw(myAnimeArea *area);
static gint cb_sprite_timeout(gpointer data);

mySprite *
my_sprite_new(myAnimeArea *area, gchar *file, gint timeout, GSourceFunc init,GSourceFunc move)
{
	//GError *error = (GError *)g_new0(GError, 1);
	mySprite *sprite = g_new0(mySprite, 1);

	sprite->anime_area = area;
	sprite->move = move;
	//sprite->pixbuf = gdk_pixbuf_new_from_file(file, &error);
	sprite->pixbuf = gdk_pixbuf_new_from_file(file, NULL);
	sprite->width = gdk_pixbuf_get_width(sprite->pixbuf);
	sprite->height = gdk_pixbuf_get_height(sprite->pixbuf);

	// printf("my_sprite_new begin %s (%d, %d)\n", file, sprite->width, sprite->height);  

	(*init)((gpointer)sprite);

	sprite->timeout = my_timeout_add(timeout, (GSourceFunc)cb_sprite_timeout, sprite);
	area->sprites = g_list_prepend(area->sprites, sprite);
	// printf("my_sprite_new end\n");
	return sprite;
}

/* スプライトを動かす (callback) */
static gint
cb_sprite_timeout(gpointer data)
{
	my_sprite_move((mySprite *) data);
	return TRUE;
}

/* ドローイングエリアにスプライトを描く */
static void
my_sprites_draw(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	gpointer sprite;

	while(sprites!=NULL)
	{
		sprite =  sprites->data;
		my_sprite_draw((mySprite *) sprites->data);
		sprites = g_list_next(sprites);
	}
}

void
my_sprite_move(mySprite *self)
{
	GSourceFunc movef = self->move;

	(*movef)((gpointer)self);
}

void
my_sprite_remove(mySprite *sprite)
{
	if(sprite->timeout) g_source_remove(sprite->timeout);
	{
		myAnimeArea *area = sprite->anime_area;
		GList *sprites = area->sprites;
		area->sprites = g_list_remove(sprites, (gpointer) sprite);
		g_object_unref((gpointer) sprite->pixbuf);
		g_free((gpointer) sprite);
	}
}

/* ドローイングエリアにスプライトを描く */
static void
my_sprite_draw(mySprite *sprite)
{
	GtkWidget *drawing_area = (sprite->anime_area)->drawing_area;

	/* スプライト用ピクスバッファをドローイングエリアに描く */
	gdk_draw_pixbuf(drawing_area->window,
			drawing_area->style->fg_gc[gtk_widget_get_state(drawing_area)],
			sprite->pixbuf,
			0, 0, sprite->x, sprite->y,
			sprite->width, sprite->height,
			GDK_RGB_DITHER_NORMAL, 0, 0);
}

/* ドローイングエリアを更新 */
static void
my_anime_area_draw(myAnimeArea *area)
{
	static GdkRectangle rect = {0, 0, 0, 0};

	rect.width = area->width;
	rect.height = area->height;

	gdk_window_begin_paint_rect(area->drawing_area->window, &rect);

	/* セットしてある背景でドローイイングエリアをクリア */
	if (area->bg_pixmap!=NULL) 
	{
		gdk_window_clear(area->drawing_area->window);
	}

	/* ドローイングエリアにスプライト達を描く */
	my_sprites_draw(area);

	/* ダブルバッファ処理の終了 */
	gdk_window_end_paint(area->drawing_area->window);
}

static gint
cb_anime_area_timeout(gpointer data)
{
	my_anime_area_draw((myAnimeArea *)data);
	return TRUE;
}

static gint
cb_anime_area_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	my_anime_area_draw((myAnimeArea *)data);
	return FALSE;
}

/* configureイベントのコールバック */
static gint
cb_anime_area_configure(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	myAnimeArea *area = (myAnimeArea *) data;

	/* ピックスマップをドローイングエリアの背景に設定 */
	if (area->bg_pixmap!=NULL)
	{
		gdk_window_set_back_pixmap(widget->window, area->bg_pixmap, FALSE);
	}
	return FALSE;
}

void
my_anime_area_remove(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	mySprite *sprite;

	while((sprite=(mySprite *)g_list_first(sprites))!=NULL)
	{
		my_sprite_remove(sprite);
	}
	if(area->timeout) g_source_remove(area->timeout);
}

myAnimeArea *
my_anime_area_new(gint width, gint height, GdkPixmap *back, gint timeout)
{
	myAnimeArea *area = (myAnimeArea *) g_new0(myAnimeArea, 1);

	// printf("anime_area_new begins\n");
	area->sprites = (GList *) NULL;
	area->bg_pixmap = back;
	area->width = width;
	area->height = height;
	area->drawing_area = gtk_drawing_area_new();
	// printf("anime_area_new begins (2)\n");
	gtk_widget_show(area->drawing_area);
	gtk_widget_set_size_request(GTK_WIDGET(area->drawing_area), 
			area->width, area->height);
	/* configureイベントのコールバックを設定
	   ドローイングエリア生成時に背景を設定する */
	// printf("anime_area_new begins (3)\n");
	gtk_widget_set_events(area->drawing_area, GDK_STRUCTURE_MASK);
	// printf("anime_area_new begins (4)\n");
	/* 最後の引数はコールバックの第3引数になる */
	g_signal_connect(G_OBJECT(area->drawing_area), "configure_event",
			G_CALLBACK(cb_anime_area_configure), (gpointer)area);
	// printf("anime_area_new begins (5)\n");
	g_signal_connect(G_OBJECT(area->drawing_area), "expose_event",
			G_CALLBACK(cb_anime_area_expose), (gpointer)area);
	// printf("anime_area_new begins (6)\n");
	area->timeout = my_timeout_add(timeout, (GSourceFunc)cb_anime_area_timeout, (gpointer) area);
	// printf("anime_area_new ends\n");
	return area;
}

