// $Id: ball.c 6991 2010-12-02 04:10:59Z suzuki $
// -*-: coding:utf-8; -*-

# include <enemy.h>
# include <anime.h>
# include <bullet.h>
# include <music.h>
# include <map.h>
# include <stdlib.h>
# include <system.h>

static void enemy_sprite_draw(enemySprite *sprite);
static void enemy_anime_area_draw(myAnimeArea *area);
static gint enemy_cb_sprite_timeout(gpointer data);

gint g_enemy_type = 0;
gint g_enemy_num = 0;
gint g_enemy_call_max = 0;
gint g_enemy_call_count = 0;
gint g_enemy_group_max = 0;
gint g_enemy_group_count = 0;
gint g_enemy_max = 0;
GList *enemy = NULL;
// pics, reward, speed, life, damage
enemyType enemy_type[KINDS_OF_ENEMYS] =  {
	{"../data/virus.png", 20, 2, 1, 1},
	{"../data/smoke.png", 20, 2, 1, 1},
	{"../data/slime.png", 40, 2, 2, 2},
	{"../data/bat.png", 30, 2, 3, 2},
	{"../data/dragon.png", 80, 2, 5, 5},
	{"../data/Bganchan.png", 100, 2, 10, 10}
};

void enemy_read_data(char *filename)
{
	FILE *fp;
	int i;
	int type;
	
	fp = fopen(filename, "r");
	if (fp == NULL) {
		printf("%s is not found.\n", filename);
		exit(0);
	}

	if (g_enemy_group_max == 0) {
		while (fscanf(fp, "%d,%d", &g_enemy_call_max, &g_enemy_type) != EOF) {
			g_enemy_group_max++;
			g_enemy_max += g_enemy_call_max;
		}
	}
	i = 0;
	while (fscanf(fp, "%d,%d", &g_enemy_call_max, &g_enemy_type) != EOF) {
		if (i == g_enemy_group_count) {
			i++;
			break;
		}
		else {
			i++;
		}
	}
	// printf("g_enemy_group_max = %d\n", g_enemy_group_max);
	// printf("type = %d\n", g_enemy_type);

	fclose(fp);
}

void
enemy_new(myAnimeArea *area)
{
	mySprite *s;

	if (g_enemy_group_count >= g_enemy_group_max) {
		return;
	}
	enemy_read_data("../data/enemy.csv");
	s = enemy_sprite_new(area, enemy_type[g_enemy_type].pics, 30,
						 (GSourceFunc)enemy_init,
						 (GSourceFunc)enemy_move);
	g_enemy_num++;
	enemy = g_list_append(enemy, (gpointer)s);
	play_se("../data/appear.wav");
	// printf("enemy_new\n");
}

// スプライトの初期座標とベクトルの設定 
void
enemy_init(enemySprite *sprite)
{
	myAnimeArea *area = sprite->anime_area;
	enemyType *type = &enemy_type[g_enemy_type];

	sprite->x = 0;
	sprite->y = area->height/2 - sprite->height/2;

	sprite->reward = type->reward;
	sprite->dx = type->speed;
	sprite->life = type->life;
	sprite->damage = type->damage;
	// printf("enemy_init\n");
}

void
enemy_move(enemySprite *sprite)
{
	myAnimeArea *area = sprite->anime_area;
	gint x = sprite->x,   y = sprite->y;
	gint dx = sprite->dx, dy = sprite->dy;
	gint width = sprite->width;
	gint height = sprite->height;

	gint i;
	gint enemy_num = 0;
	enemySprite *e;
	
	/* スプライトの座標を更新 */
	x += dx; y += dy;
	
 	sprite->x = x;   sprite->y = y;
	sprite->dx = dx; sprite->dy = dy;

	enemy_num = (gint)g_list_length(enemy);
	//printf("enemy_num = %d\n", enemy_num);
	for (i = 0; i < enemy_num; i++) {
		e = (enemySprite *)g_list_nth_data(enemy, i);
		//printf("e->reward = %d\n", e->reward);
		if (enemy_collision(e)) {
			if (e->life <= 0) {
				g_system->money += e->reward;
				system_draw(g_system->life, g_system->money, FALSE, 1);
				enemy_remove(e);
				enemy = g_list_delete_link(enemy, g_list_nth(enemy, i));
				break;
			}
		}
		if(e->x > area->width - width && e->life > 0) {
			// printf("damage %d\n", sprite->damage);
			g_system->life -= e->damage;
			system_draw(g_system->life, g_system->money, FALSE, 0);
			if (g_system->life <= 0) {
				stop_music();
				printf(" *****    *****   *     *  *******   *****   *      *  *******  ****** \n");
				printf("*        *     *  **   **  *        *     *  *      *  *        *     *\n");
				printf("*   ***  *******  * * * *  *****    *     *   *    *   *****    *******\n");
				printf("*     *  *     *  *  *  *  *        *     *    *  *    *        *    * \n");
				printf(" *****   *     *  *     *  *******   *****      *      *******  *     *\n");
				play_se("../data/gameover.wav");
				sleep(3);
				exit(0);
			}
			enemy_remove(e);
			enemy = g_list_delete_link(enemy, g_list_nth(enemy, i));
			play_se("../data/goal.wav");
			break;
		}
	}
}

gint
enemy_collision(enemySprite *self)
{
	gint i;
	gint bullet_num = (gint)g_list_length(bullet);
	bulletSprite *b;	

	for (i = 0; i < bullet_num; i++) {
		b = (bulletSprite *)g_list_nth_data(bullet, i);
		myAnimeArea *area = b->anime_area;
		gint x = b->x;
		gint y = b->y;
		gint width = b->width;
		gint height = b->height;

		gint lx = abs(self->x - b->x);
		gint ly = abs(self->y - b->y);
		gint wx = (self->width + b->width)/2;
		gint wy = (self->height + b->height)/2;
		
		if (lx<wx && ly<wy)	{
			bullet_remove(b);
			bullet = g_list_delete_link(bullet, g_list_nth(bullet, i));
			self->life -= b->damage;
			play_se("../data/explosion.wav");
			return TRUE;
		}
		else if (x < 0 || y < 0 || x > area->width - width || y > area->height - height) {
			bullet_remove(b);
			bullet = g_list_delete_link(bullet, g_list_nth(bullet, i));
			play_se("../data/goal.wav");
			return FALSE;
		}
	}
	return FALSE;
}

void
enemy_remove(enemySprite *self)
{
	my_sprite_remove(self);
	g_enemy_max--;
	if (g_enemy_max == 0 && g_system->life > 0) {
		stop_music();
		printf(" *****   *        *******   *****   ******     ***\n");
		printf("*     *  *        *        *     *  *     *    ***\n");
		printf("*        *        *****    *******  *******    ***\n");
		printf("*     *  *        *        *     *  *    *        \n");
		printf(" *****   *******  *******  *     *  *     *    ***\n");
		play_se("../data/gameclear.wav");
		sleep(3);
		exit(0);
	}
	// printf("enemy_remove\n");
}

enemySprite *
enemy_sprite_new(myAnimeArea *area, gchar *file, gint timeout, GSourceFunc init,GSourceFunc move)
{
  //GError *error = (GError *)g_new0(GError, 1);
	enemySprite *sprite = g_new0(enemySprite, 1);

	sprite->anime_area = area;
	sprite->move = move;
	//sprite->pixbuf = gdk_pixbuf_new_from_file(file, &error);
	sprite->pixbuf = gdk_pixbuf_new_from_file(file, NULL);
	sprite->width = gdk_pixbuf_get_width(sprite->pixbuf);
	sprite->height = gdk_pixbuf_get_height(sprite->pixbuf);

	// printf("my_sprite_new begin %s (%d, %d)\n", file, sprite->width, sprite->height);  

	(*init)((gpointer)sprite);

	sprite->timeout = my_timeout_add(timeout, (GSourceFunc)enemy_cb_sprite_timeout, sprite);
	area->sprites = g_list_prepend(area->sprites, sprite);
	// printf("my_sprite_new end\n");
	return sprite;
}

/* スプライトを動かす (callback) */
static gint
enemy_cb_sprite_timeout(gpointer data)
{
	enemy_sprite_move((mySprite *) data);
	return TRUE;
}

/* ドローイングエリアにスプライトを描く */
static void
enemy_sprites_draw(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	gpointer sprite;

	while(sprites!=NULL)
	{
		sprite =  sprites->data;
		enemy_sprite_draw((mySprite *) sprites->data);
		sprites = g_list_next(sprites);
	}
}

void
enemy_sprite_move(enemySprite *self)
{
	GSourceFunc movef = self->move;

	(*movef)((gpointer)self);
}

void
enemy_sprite_remove(enemySprite *sprite)
{
	if(sprite->timeout) g_source_remove(sprite->timeout);
	{
		myAnimeArea *area = sprite->anime_area;
		GList *sprites = area->sprites;
		area->sprites = g_list_remove(sprites, (gpointer) sprite);
		g_object_unref((gpointer) sprite->pixbuf);
		g_free((gpointer) sprite);
	}
}

/* ドローイングエリアにスプライトを描く */
static void
enemy_sprite_draw(enemySprite *sprite)
{
	GtkWidget *drawing_area = (sprite->anime_area)->drawing_area;

	/* スプライト用ピクスバッファをドローイングエリアに描く */
	gdk_draw_pixbuf(drawing_area->window,
			drawing_area->style->fg_gc[gtk_widget_get_state(drawing_area)],
			sprite->pixbuf,
			0, 0, sprite->x, sprite->y,
			sprite->width, sprite->height,
			GDK_RGB_DITHER_NORMAL, 0, 0);
}

/* ドローイングエリアを更新 */
static void
enemy_anime_area_draw(myAnimeArea *area)
{
	static GdkRectangle rect = {0, 0, 0, 0};

	rect.width = area->width;
	rect.height = area->height;

	gdk_window_begin_paint_rect(area->drawing_area->window, &rect);

	/* セットしてある背景でドローイイングエリアをクリア */
	if (area->bg_pixmap!=NULL) 
	{
		gdk_window_clear(area->drawing_area->window);
	}

	/* ドローイングエリアにスプライト達を描く */
	enemy_sprites_draw(area);

	/* ダブルバッファ処理の終了 */
	gdk_window_end_paint(area->drawing_area->window);
}

static gint
enemy_cb_anime_area_timeout(gpointer data)
{
	enemy_anime_area_draw((myAnimeArea *)data);
	return TRUE;
}

static gint
enemy_cb_anime_area_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	enemy_anime_area_draw((myAnimeArea *)data);
	return FALSE;
}

/* configureイベントのコールバック */
static gint
enemy_cb_anime_area_configure(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	myAnimeArea *area = (myAnimeArea *) data;

	/* ピックスマップをドローイングエリアの背景に設定 */
	if (area->bg_pixmap!=NULL)
	{
		gdk_window_set_back_pixmap(widget->window, area->bg_pixmap, FALSE);
	}
	return FALSE;
}

void
enemy_anime_area_remove(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	enemySprite *sprite;

	while((sprite=(mySprite *)g_list_first(sprites))!=NULL)
	{
		enemy_sprite_remove(sprite);
	}
	if(area->timeout) g_source_remove(area->timeout);
}

myAnimeArea *
enemy_anime_area_new(gint width, gint height, GdkPixmap *back, gint timeout)
{
	myAnimeArea *area = (myAnimeArea *) g_new0(myAnimeArea, 1);

	// printf("anime_area_new begins\n");
	area->sprites = (GList *) NULL;
	area->bg_pixmap = back;
	area->width = width;
	area->height = height;
	area->drawing_area = gtk_drawing_area_new();
	// printf("anime_area_new begins (2)\n");
	gtk_widget_show(area->drawing_area);
	gtk_widget_set_size_request(GTK_WIDGET(area->drawing_area), 
			area->width, area->height);
	/* configureイベントのコールバックを設定
	   ドローイングエリア生成時に背景を設定する */
	// printf("anime_area_new begins (3)\n");
	gtk_widget_set_events(area->drawing_area, GDK_STRUCTURE_MASK);
	// printf("anime_area_new begins (4)\n");
	/* 最後の引数はコールバックの第3引数になる */
	g_signal_connect(G_OBJECT(area->drawing_area), "configure_event",
			G_CALLBACK(enemy_cb_anime_area_configure), (gpointer)area);
	// printf("anime_area_new begins (5)\n");
	g_signal_connect(G_OBJECT(area->drawing_area), "expose_event",
			G_CALLBACK(enemy_cb_anime_area_expose), (gpointer)area);
	// printf("anime_area_new begins (6)\n");
	area->timeout = my_timeout_add(timeout, (GSourceFunc)enemy_cb_anime_area_timeout, (gpointer) area);
	// printf("anime_area_new ends\n");
	return area;
}
