// $Id: gtk_lib.c 6991 2010-12-02 04:10:59Z suzuki $
// -*- coding: utf-8; -*- 
// Time-stamp: <2010-11-16 17:58:01 suzuki>

# include <gtk/gtk.h>
# include "gtk_lib.h"

static GList *registered_timeouts = NULL;

// ウィンドウを閉じる際に呼ばれる関数
gint my_close_handler(GtkWidget *widget, gpointer gdata);

gint 
my_timeout_add(gint timeout, GSourceFunc handler, gpointer widget)
{
  gint timeout_no = g_timeout_add(timeout, handler, (gpointer)widget);
  //g_signal_connect(G_OBJECT(widget), "delete_event",
  //		       G_CALLBACK(my_close_handler), (gpointer) &registered_timeouts);
  registered_timeouts = g_list_prepend(registered_timeouts, (gpointer)timeout_no);
  return timeout_no;
}

gint 
my_timeout_remove(gint timeout_no)
{
  g_source_remove((gint) timeout_no);
  registered_timeouts =
    g_list_remove(registered_timeouts, (gpointer) timeout_no);
  return timeout_no;
}
      
// ボタンがクリックされた時に呼ばれる関数
void my_button_clicked_handler(GtkWidget *widget, gpointer data);

void 
my_init(int argc, char *argv[])
{
  // 初期化： ロケール設定
  gtk_set_locale();

  // 初期化： gtk オプションの処理
  gtk_init( &argc, &argv );

  // 初期化： rc ファイルの読み込み
  gtk_rc_parse("./gtkrc");
}

//Window
GtkWidget *
my_window_new(gint width, gint height)
{
  GtkWidget *window;

  //トップレベルウィンドウを生成 */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  //ウィンドウを消す時に呼ばれる関数を登録
  g_signal_connect(GTK_OBJECT(window), "delete_event",
		   G_CALLBACK(my_close_handler), NULL);

  //コンテナ内のオブジェクトの周りにすき間を作成
  gtk_container_set_border_width(GTK_CONTAINER(window), 15);
  
  //ウィンドウを可視状態にする
  gtk_widget_show(window);

  //ウィンドウの大きさを変更
  gtk_window_resize(GTK_WINDOW(window), width, height);
  gtk_widget_show(window);

  return window;
}

// ラベル
GtkWidget *
my_label_new(char *label, My_Font_Desc *fd)
{
  GtkWidget * lbl = gtk_label_new(label); 

  gtk_widget_modify_font(GTK_WIDGET(lbl), fd);
  gtk_widget_show(lbl);
  return lbl;
}

//ボタン
GtkWidget *
my_button_new(char *label, My_Font_Desc *fd)
{
  GtkWidget * lbl = gtk_label_new(label); 
  GtkWidget * button;

  lbl = gtk_label_new(label); 
  gtk_widget_modify_font(GTK_WIDGET(lbl), fd);  
  gtk_widget_show(lbl);
  button = gtk_button_new();   
  gtk_container_add(GTK_CONTAINER(button), lbl);
  //click処理関数をセット
  g_signal_connect(GTK_OBJECT(button), "clicked",
		   G_CALLBACK(my_button_clicked_handler), (gpointer *)label);
  gtk_widget_show(button);

  return button;
}

//画像ファイルを指定して，イメージを作成
GtkWidget *
my_image_new(char *path)
{
  GtkWidget *image = gtk_image_new();
  gtk_image_set_from_file((GtkImage *)image, path);
  return image;
}

static gchar *defaultfontname = "IPA Gothic 32";

My_Font_Desc  *
my_font_description(const gchar *fontname)
{
  if (!*fontname)
    { 
      fontname = defaultfontname;
    }
  return pango_font_description_from_string(fontname);
}

// イベント処理ハンドラ（関数）

gint my_close_handler(GtkWidget *widget, gpointer gdata)
{
  // g_print("Quitting ... \n");
  {
    // timeout ハンドラを消す
    
    while (registered_timeouts!=NULL)
      {
	g_source_remove((gint) registered_timeouts->data);
	registered_timeouts = g_list_next(registered_timeouts);
      }
  }
  gtk_main_quit();
  /* 終了作業を継続 */
  return(FALSE);
}

// ボタンを押した時の処理関数
static gboolean click_and_remove = FALSE;

gboolean my_set_click_and_remove (gboolean val)
{
  gboolean rval = click_and_remove;

  click_and_remove = val;
  return rval;
}

void my_button_clicked_handler(GtkWidget *widget, gpointer data)
{
  // g_print("[%s] button is pressed.\n", (char *)data);
  if (click_and_remove)
    {
      my_widget_remove(widget);
    }
}

// widgetを消す関数

void my_widget_remove(GtkWidget *widget)
{
  GtkWidget *parent = gtk_widget_get_parent(widget);
  GList *list;

  list = gtk_container_get_children(GTK_CONTAINER(parent));
  list = g_list_remove(list, (gpointer) widget);
  //g_object_unref(widget);
  gtk_widget_destroy(widget);
}


GdkPixmap *my_pixmap_new_from_data(GtkWidget *window, char *path)
{
  GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(path, NULL);
  int w = gdk_pixbuf_get_width(pixbuf);
  int h = gdk_pixbuf_get_height(pixbuf);
  GdkPixmap *pixmap = gdk_pixmap_new(window->window, w, h, -1);
  GdkBitmap *mask;

  gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 0);
  gdk_window_shape_combine_mask(window->window, mask, 0, 0);
  //gdk_draw_pixbuf(pixmap,
  //                window->style->fg_gc[gtk_widget_get_state(window)],
  //                pixbuf,
  //                0, 0, 0, 0, -1, -1,
  //                GDK_RGB_DITHER_NORMAL, 0, 0);

  //g_object_unref(pixbuf);
  return pixmap;
}

GtkWidget *my_drawing_area_new(int w, int h, GdkPixmap *pixmap, gboolean relative)
{
  GtkWidget *drawing_area = gtk_drawing_area_new();

  gtk_widget_set_size_request(drawing_area, w, h);
  gtk_widget_show(drawing_area);
  //gdk_window_set_back_pixmap(drawing_area->window, pixmap, relative);
  return drawing_area;
}
