#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <map.h>
#include <system.h>

static void system_sprite_draw(systemSprite *sprite);
static void system_anime_area_draw(myAnimeArea *area);
static gint system_cb_sprite_timeout(gpointer data);

System *g_system;
GList *system_list = NULL;
gboolean g_system_delete_life[9] = { FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE };
gint g_count_life = 0;

void systems_init()
{
	int i;

	g_system = (System *)malloc(sizeof(System));
	system_load();
	system_draw(g_system->life, g_system->money, TRUE, -1);
}

void system_save()
{
	FILE *fp1;
	char *filename = "Save_data";
	if(!SYSTEM_DEBUG)
	{
	  fp1 = fopen(filename,"w");
	}
}

void system_load()
{
	FILE *fp;
	char *file = "../data/system.csv"; /* セーブデータファイル */

	fp = fopen(file,"r");
	if(fp == NULL)
	{
	  printf("Can't load file system.csv .\n");
	  g_system->life = 10;
	  g_system->money = 1000;
	}
	else
	{
	  fscanf(fp,"%d,%d",&g_system->life,&g_system->money);
	  fclose(fp);
	}
}

void system_draw(gint life, gint money, gboolean first, gint type)
{
	int i;
	int	work;
	int num = 0;

	if (first == FALSE) {
		if (type == 0 || type == -1) {
			g_count_life = 0;
			for (i = 0; i < 3; i++) {
				g_system_delete_life[i] = TRUE;
			}
			work = life;
			for (i = 0; i < 3; i++) {
				if (work > 0) {
					num = work % 10;
					work /= 10;
				}
				else {
					num = 0;
				}
				system_new(area, (2-i) * 32, 0, num);
			}
		}
		if (type == 1 || type == -1) {
			g_count_life = 3;
			for (i = 3; i < 9; i++) {
				g_system_delete_life[i] = TRUE;
			}
			work = money;
			for (i = 0; i < 6; i++) {
				if (work > 0) {
					num = work % 10;
					work /= 10;
				}
				else {
					num = 0;
				}
				system_new(area, area->width - (i+1)*32, 0, num);
			} 
		}
	}
	else {
		work = life;
		for (i = 0; i < 3; i++) {
			if (work > 0) {
				num = work % 10;
				work /= 10;
			}
			else {
				num = 0;
			}
			system_new(area, (2-i) * 32, 0, num);
		}
		work = money;
		for (i = 0; i < 6; i++) {
			if (work > 0) {
				num = work % 10;
				work /= 10;
			}
			else {
				num = 0;
			}
			system_new(area, area->width - (i+1)*32, 0, num);
		} 
	}
}

void system_set(int goflag,int cost)
{
	/*
	   味方及び敵の配置を行う関数
	   引数：
	   ・マップの通行許可 int goflag
	   ・味方のコスト int cost
	 */
	//waveにより敵の種類が決まっている。
	if(goflag == 0)
	{
		printf("Can't put here.\n");
	}
}

void
system_new(myAnimeArea *area, gint x, gint y, gint num)
{
	systemSprite *s;
	char filename[256] = "../data/";
	char num_to_str[2];
	sprintf(num_to_str, "%d", num);
	strcat(filename, num_to_str);
	strcat(filename, ".png");
	// printf("CALL: %s\n", filename);

	s = system_sprite_new(area, filename,
						  1,
						  (GSourceFunc) system_init,
						  (GSourceFunc) system_move,
						  x, y);

	system_list = g_list_append(system_list, (gpointer)s);
}

// スプライトの初期座標とベクトルの設定 
void
system_init(systemSprite *sprite)
{
}

void 
system_move(systemSprite *sprite)
{
	if (g_system_delete_life[sprite->num] == TRUE) {
		system_remove(sprite);
		g_system_delete_life[sprite->num] = FALSE;
	}
}

void system_remove(systemSprite *self)
{
	system_sprite_remove(self);
}

systemSprite *
system_sprite_new(myAnimeArea *area, gchar *file, gint timeout, GSourceFunc init,GSourceFunc move, gint x, gint y)
{
	//GError *error = (GError *)g_new0(GError, 1);
	systemSprite *sprite = g_new0(systemSprite, 1);

	sprite->anime_area = area;
	sprite->move = move;
	//sprite->pixbuf = gdk_pixbuf_new_from_file(file, &error);
	sprite->pixbuf = gdk_pixbuf_new_from_file(file, NULL);
	sprite->width = gdk_pixbuf_get_width(sprite->pixbuf);
	sprite->height = gdk_pixbuf_get_height(sprite->pixbuf);
	sprite->x = x;
	sprite->y = y;

	sprite->num = g_count_life;
	g_count_life++;
	if (g_count_life >= 9) g_count_life = 0;

	// printf("my_sprite_new begin %s (%d, %d)\n", file, sprite->width, sprite->height);  

	(*init)((gpointer)sprite);

	sprite->timeout = my_timeout_add(timeout, (GSourceFunc)system_cb_sprite_timeout, sprite);
	area->sprites = g_list_prepend(area->sprites, sprite);
	return sprite;
}

/* スプライトを動かす (callback) */
static gint
system_cb_sprite_timeout(gpointer data)
{
	system_sprite_move((systemSprite *) data);
	return TRUE;
}

/* ドローイングエリアにスプライトを描く */
static void
system_sprites_draw(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	gpointer sprite;

	while(sprites!=NULL)
	{
		sprite =  sprites->data;
		system_sprite_draw((systemSprite *) sprites->data);
		sprites = g_list_next(sprites);
	}
}

void
system_sprite_move(systemSprite *self)
{
	GSourceFunc movef = self->move;

	(*movef)((gpointer)self);
}

void
system_sprite_remove(systemSprite *sprite)
{
	if(sprite->timeout) g_source_remove(sprite->timeout);
	{
		myAnimeArea *area = sprite->anime_area;
		GList *sprites = area->sprites;
		area->sprites = g_list_remove(sprites, (gpointer) sprite);
		g_object_unref((gpointer) sprite->pixbuf);
		g_free((gpointer) sprite);
	}
}

/* ドローイングエリアにスプライトを描く */
static void
system_sprite_draw(systemSprite *sprite)
{
	GtkWidget *drawing_area = (sprite->anime_area)->drawing_area;

	/* スプライト用ピクスバッファをドローイングエリアに描く */
	gdk_draw_pixbuf(drawing_area->window,
			drawing_area->style->fg_gc[gtk_widget_get_state(drawing_area)],
			sprite->pixbuf,
			0, 0, sprite->x, sprite->y,
			sprite->width, sprite->height,
			GDK_RGB_DITHER_NORMAL, 0, 0);
}

/* ドローイングエリアを更新 */
static void
system_anime_area_draw(myAnimeArea *area)
{
static GdkRectangle rect = {0, 0, 0, 0};

	rect.width = area->width;
	rect.height = area->height;

	gdk_window_begin_paint_rect(area->drawing_area->window, &rect);

	/* セットしてある背景でドローイイングエリアをクリア */
	if (area->bg_pixmap!=NULL) 
	{
		gdk_window_clear(area->drawing_area->window);
	}

	/* ドローイングエリアにスプライト達を描く */
	system_sprites_draw(area);

	/* ダブルバッファ処理の終了 */
	gdk_window_end_paint(area->drawing_area->window);
}

static gint
system_cb_anime_area_timeout(gpointer data)
{
	system_anime_area_draw((myAnimeArea *)data);
	return TRUE;
}

static gint
system_cb_anime_area_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	system_anime_area_draw((myAnimeArea *)data);
	return FALSE;
}

/* configureイベントのコールバック */
static gint
system_cb_anime_area_configure(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	myAnimeArea *area = (myAnimeArea *) data;

	/* ピックスマップをドローイングエリアの背景に設定 */
	if (area->bg_pixmap!=NULL)
	{
		gdk_window_set_back_pixmap(widget->window, area->bg_pixmap, FALSE);
	}
	return FALSE;
}

void system_anime_area_remove(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	systemSprite *sprite;

	while((sprite=(systemSprite *)g_list_first(sprites))!=NULL)
	{
		system_sprite_remove(sprite);
	}
	if(area->timeout) g_source_remove(area->timeout);
}

myAnimeArea *
system_anime_area_new(gint width, gint height, GdkPixmap *back, gint timeout)
{
	myAnimeArea *area = (myAnimeArea *) g_new0(myAnimeArea, 1);

	// printf("anime_area_new begins\n");
	area->sprites = (GList *) NULL;
	area->bg_pixmap = back;
	area->width = width;
	area->height = height;
	area->drawing_area = gtk_drawing_area_new();
	// printf("anime_area_new begins (2)\n");
	gtk_widget_show(area->drawing_area);
	gtk_widget_set_size_request(GTK_WIDGET(area->drawing_area), 
			area->width, area->height);
	/* configureイベントのコールバックを設定
	   ドローイングエリア生成時に背景を設定する */
	// printf("anime_area_new begins (3)\n");
	gtk_widget_set_events(area->drawing_area, GDK_STRUCTURE_MASK);
	// printf("anime_area_new begins (4)\n");
	/* 最後の引数はコールバックの第3引数になる */
	g_signal_connect(G_OBJECT(area->drawing_area), "configure_event",
			G_CALLBACK(system_cb_anime_area_configure), (gpointer)area);
	// printf("anime_area_new begins (5)\n");
	g_signal_connect(G_OBJECT(area->drawing_area), "expose_event",
			G_CALLBACK(system_cb_anime_area_expose), (gpointer)area);
	// printf("anime_area_new begins (6)\n");
	area->timeout = my_timeout_add(timeout, (GSourceFunc)system_cb_anime_area_timeout, (gpointer) area);
	// printf("anime_area_new ends\n");
	return area;
}
