// $Id: ball.c 6991 2010-12-02 04:10:59Z suzuki $
// -*-: coding:utf-8; -*-

# include <unit.h>
# include <map.h>
# include <anime.h>
# include <enemy.h>
# include <music.h>
# include <bullet.h>

static void unit_sprite_draw(unitSprite *sprite);
static void unit_anime_area_draw(myAnimeArea *area);
static gint unit_cb_sprite_timeout(gpointer data);

gint g_unit_num = 0;
GList *unit_list = NULL;
/* 画像, コスト, 索敵, 射撃間隔, 撃つ弾番号, レベルアップ先 */
unitType unit_type[KINDS_OF_UNITS] = {
  {"../data/ganchan.png", 100, 1.5, 5000, 0, 1}, //0
  {"../data/GreenBall.png", 200, 2, 3000, 1, 2}, //1
  {"../data/YellowBall.png", 300, 2.5, 2000, 2, -1}, //2
  {"../data/keibiin_blue.png", 100, 2, 6000, 3, 4}, //3
  {"../data/keibiin_green.png", 200, 2.5, 5000, 4, 5}, //4
  {"../data/keibiin_red.png", 300, 3, 4000, 5, -1} //5
};

	void
unit_new(myAnimeArea *area, gint x, gint y, gint type)
{
	unitSprite *s;

	// printf("unit_new (%d)\n", g_unit_num);

	s = unit_sprite_new(area, unit_type[type].pics, 
			unit_type[type].fire_interval,
			(GSourceFunc) unit_init,
			(GSourceFunc)unit_move,
			x, y);

	s->level = 0;
	s->cost = unit_type[type].cost;
	s->unit_type = type;
	s->range = unit_type[type].range * DIVIDE_SIZE;
	s->bullet_type = unit_type[type].bullet_type;
	s->unit_newtype = unit_type[type].unit_newtype;

	unit_list = g_list_append(unit_list, (gpointer)s);

	play_se("../data/set_unit.wav");

	g_unit_num++;
}

// スプライトの初期座標とベクトルの設定 
	void
unit_init(unitSprite *sprite)
{
	/*
	   unitType *type = &unit_type[g_unit_num%KINDS_OF_UNITS];

	   sprite->level = 0;
	   sprite->cost = type->cost;
	   sprite->bullet_type = type->bullet_type;
	 */
}

	void 
unit_move(unitSprite *sprite)
{
	myAnimeArea *area = sprite->anime_area;
	//	gint width = sprite->width;
	//	gint height = sprite->height;
	gtk_vector bullet_vector;

	bullet_vector = unit_fire_control(sprite);

	if(bullet_vector.x != 100000)
		bullet_new(area,
				sprite->x + sprite->width/2,
				sprite->y + sprite->height/2,
				bullet_vector.x,
				bullet_vector.y,
				sprite->bullet_type);

}

void unit_remove(unitSprite *self)
{
	g_unit_num--;
	unit_sprite_remove(self);
}

/*
   gint
   unit_is_clicked(gpointer me)
   {
   myAnimeArea *b = (unitSprite *)me;

   printf("unit_is_clicked!\n");
   unit_new(b, 400, 10);
   return FALSE;
   }
 */

	unitSprite *
unit_sprite_new(myAnimeArea *area, gchar *file, gint timeout, GSourceFunc init,GSourceFunc move, gint x, gint y)
{
	//GError *error = (GError *)g_new0(GError, 1);
	unitSprite *sprite = g_new0(unitSprite, 1);

	sprite->anime_area = area;
	sprite->move = move;
	//sprite->pixbuf = gdk_pixbuf_new_from_file(file, &error);
	sprite->pixbuf = gdk_pixbuf_new_from_file(file, NULL);
	sprite->width = gdk_pixbuf_get_width(sprite->pixbuf);
	sprite->height = gdk_pixbuf_get_height(sprite->pixbuf);

	/* 追加 */
	sprite->x = x - sprite->width / 2;
	sprite->y = y - sprite->height / 2;
	/* 追加ここまで */

	// printf("my_sprite_new begin %s (%d, %d)\n", file, sprite->width, sprite->height);  

	(*init)((gpointer)sprite);

	sprite->timeout = my_timeout_add(timeout, (GSourceFunc)unit_cb_sprite_timeout, sprite);
	area->sprites = g_list_prepend(area->sprites, sprite);
	// printf("my_sprite_new end\n");
	return sprite;
}

/* スプライトを動かす (callback) */
	static gint
unit_cb_sprite_timeout(gpointer data)
{
	unit_sprite_move((unitSprite *) data);
	return TRUE;
}

/* ドローイングエリアにスプライトを描く */
	static void
unit_sprites_draw(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	gpointer sprite;

	while(sprites!=NULL)
	{
		sprite =  sprites->data;
		unit_sprite_draw((unitSprite *) sprites->data);
		sprites = g_list_next(sprites);
	}
}

	void
unit_sprite_move(unitSprite *self)
{
	GSourceFunc movef = self->move;

	(*movef)((gpointer)self);
}

	void
unit_sprite_remove(unitSprite *sprite)
{
	if(sprite->timeout) g_source_remove(sprite->timeout);
	{
		myAnimeArea *area = sprite->anime_area;
		GList *sprites = area->sprites;
		area->sprites = g_list_remove(sprites, (gpointer) sprite);
		g_object_unref((gpointer) sprite->pixbuf);
		g_free((gpointer) sprite);
	}
}

/* ドローイングエリアにスプライトを描く */
	static void
unit_sprite_draw(unitSprite *sprite)
{
	GtkWidget *drawing_area = (sprite->anime_area)->drawing_area;

	/* スプライト用ピクスバッファをドローイングエリアに描く */
	gdk_draw_pixbuf(drawing_area->window,
			drawing_area->style->fg_gc[gtk_widget_get_state(drawing_area)],
			sprite->pixbuf,
			0, 0, sprite->x, sprite->y,
			sprite->width, sprite->height,
			GDK_RGB_DITHER_NORMAL, 0, 0);
}

/* ドローイングエリアを更新 */
	static void
unit_anime_area_draw(myAnimeArea *area)
{
	static GdkRectangle rect = {0, 0, 0, 0};

	rect.width = area->width;
	rect.height = area->height;

	gdk_window_begin_paint_rect(area->drawing_area->window, &rect);

	/* セットしてある背景でドローイイングエリアをクリア */
	if (area->bg_pixmap!=NULL) 
	{
		gdk_window_clear(area->drawing_area->window);
	}

	/* ドローイングエリアにスプライト達を描く */
	unit_sprites_draw(area);

	/* ダブルバッファ処理の終了 */
	gdk_window_end_paint(area->drawing_area->window);
}

	static gint
unit_cb_anime_area_timeout(gpointer data)
{
	unit_anime_area_draw((myAnimeArea *)data);
	return TRUE;
}

	static gint
unit_cb_anime_area_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	unit_anime_area_draw((myAnimeArea *)data);
	return FALSE;
}

/* configureイベントのコールバック */
	static gint
unit_cb_anime_area_configure(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	myAnimeArea *area = (myAnimeArea *) data;

	/* ピックスマップをドローイングエリアの背景に設定 */
	if (area->bg_pixmap!=NULL)
	{
		gdk_window_set_back_pixmap(widget->window, area->bg_pixmap, FALSE);
	}
	return FALSE;
}

void unit_anime_area_remove(myAnimeArea *area)
{
	GList *sprites = area->sprites;
	unitSprite *sprite;

	while((sprite=(unitSprite *)g_list_first(sprites))!=NULL)
	{
		unit_sprite_remove(sprite);
	}
	if(area->timeout) g_source_remove(area->timeout);
}

	myAnimeArea *
unit_anime_area_new(gint width, gint height, GdkPixmap *back, gint timeout)
{
	myAnimeArea *area = (myAnimeArea *) g_new0(myAnimeArea, 1);

	// printf("anime_area_new begins\n");
	area->sprites = (GList *) NULL;
	area->bg_pixmap = back;
	area->width = width;
	area->height = height;
	area->drawing_area = gtk_drawing_area_new();
	// printf("anime_area_new begins (2)\n");
	gtk_widget_show(area->drawing_area);
	gtk_widget_set_size_request(GTK_WIDGET(area->drawing_area), 
			area->width, area->height);
	/* configureイベントのコールバックを設定
	   ドローイングエリア生成時に背景を設定する */
	// printf("anime_area_new begins (3)\n");
	gtk_widget_set_events(area->drawing_area, GDK_STRUCTURE_MASK);
	// printf("anime_area_new begins (4)\n");
	/* 最後の引数はコールバックの第3引数になる */
	g_signal_connect(G_OBJECT(area->drawing_area), "configure_event",
			G_CALLBACK(unit_cb_anime_area_configure), (gpointer)area);
	// printf("anime_area_new begins (5)\n");
	g_signal_connect(G_OBJECT(area->drawing_area), "expose_event",
			G_CALLBACK(unit_cb_anime_area_expose), (gpointer)area);
	// printf("anime_area_new begins (6)\n");
	area->timeout = my_timeout_add(timeout, (GSourceFunc)unit_cb_anime_area_timeout, (gpointer) area);
	// printf("anime_area_new ends\n");
	return area;
}

gtk_vector unit_fire_control(unitSprite *sprite){
	gtk_vector ans;
	gint i;
	enemySprite *target = NULL;
	gint dist_x, dist_y;
	gint sq_dist;
	gint min_dist_x, min_dist_y;
	gint min_sq_dist = 10000000;
	enemySprite *enemy_p;


	for(i=0; i<g_list_length(enemy); i++){
		enemy_p = (enemySprite *)g_list_nth_data(enemy, i);

		if( enemy_p->life > 0 ){
			dist_x = ( enemy_p->x + enemy_p->width/2 ) - ( sprite->x + sprite->width/2 );
			dist_y = ( enemy_p->y + enemy_p->height/2 ) - ( sprite->y + sprite->width/2 );
			sq_dist = dist_x*dist_x+dist_y*dist_y ;

			if( sq_dist < min_sq_dist && sq_dist <= sprite->range * sprite->range ){

				min_dist_x = dist_x;
				min_dist_y = dist_y;
				min_sq_dist = sq_dist;
				target = enemy_p;
			}
		}
	}

	if( target == NULL ){
		ans.x = 100000;
		ans.y = 100000;
		return ans;
	}  

	ans = unit_simple_aim(sprite, min_dist_x, min_dist_y, sqrt((double)min_sq_dist));


	return ans;
}

gtk_vector unit_simple_aim(unitSprite *sprite, gint dist_x, gint dist_y, gdouble dist){
	gtk_vector ans;

	ans.x = (gint)( (double)bullet_type[sprite->bullet_type].speed * dist_x / dist );
	ans.y = (gint)( (double)bullet_type[sprite->bullet_type].speed * dist_y / dist );

	return ans;
}

gtk_vector unit_linear_prediction(gint self_x, gint self_y, enemySprite *e, gint spd){
	gtk_vector ans;
	gdouble t, t1, t2;

	gint dx = e->x - self_x;
	gint dy = e->y - self_y;

	gint A = e->dx * e->dx + e->dy * e->dy + spd * spd;
	gint B = 2 * ( dx * e->dx + dy * e->dy );
	gint C = dx * dx + dy * dy;

	gint r = B * B - 4 * A * C;

	if( r < 0 ){
		ans.x = 0;
		ans.y = 0;
		return ans;
	}

	t1 = ( -B + sqrt( (double)r ) ) / ( 2 * A );
	t2 = ( -B - sqrt( (double)r ) ) / ( 2 * A );
	t1 = ( -B + (double)r ) / ( 2 * A );
	t2 = ( -B - (double)r ) / ( 2 * A );

	if( t1 <= 0 )
		t = t2;
	else if( t2 <= 0 )
		t = t1;
	else
		t = fmin(t1, t2);

	ans.x = (gint)(dx / t + e->dx);
	ans.y = (gint)(dy / t + e->dy);
	return ans;
}

void unit_delete(gint x, gint y){
	gint i;
	gint workx, worky;
	gint pos_x = ( x / DIVIDE_SIZE ) * DIVIDE_SIZE;
	gint pos_y = ( y / DIVIDE_SIZE ) * DIVIDE_SIZE;

	for(i=0; i<g_list_length(unit_list); i++){
		workx = ((unitSprite *)g_list_nth_data(unit_list, i))->x;
		worky = ((unitSprite *)g_list_nth_data(unit_list, i))->y;

		if( workx >= pos_x && workx < pos_x + DIVIDE_SIZE &&
				worky >= pos_y && worky < pos_y + DIVIDE_SIZE ){
			g_system->money += ((unitSprite *)g_list_nth_data(unit_list, i))->cost * 0.5;
			system_draw(g_system->life, g_system->money, FALSE, 1);
			unit_remove( (unitSprite *)g_list_nth_data(unit_list, i) );
			unit_list = g_list_delete_link(unit_list, g_list_nth(unit_list, i));
			play_se("../data/sell.wav");
			return;
		}
	}
}

void unit_upgrade(myAnimeArea *area, gint x, gint y){
	gint i;
	gint workx, worky;
	gint pos_x = ( x / DIVIDE_SIZE ) * DIVIDE_SIZE;
	gint pos_y = ( y / DIVIDE_SIZE ) * DIVIDE_SIZE;

	gint newtype;

	// printf("++++++++++++++++++ 座標 %d %d\n", pos_x, pos_y);

	for(i=0; i<g_list_length(unit_list); i++){
		workx = ((unitSprite *)g_list_nth_data(unit_list, i))->x;
		worky = ((unitSprite *)g_list_nth_data(unit_list, i))->y;

		if( workx >= pos_x && workx < pos_x + DIVIDE_SIZE &&
				worky >= pos_y && worky < pos_y + DIVIDE_SIZE ){

			newtype = ((unitSprite *)g_list_nth_data(unit_list, i))->unit_newtype;

			// printf("++++++++++++++++++  このユニットのレベルアップ先は %d\n", newtype);

			if( newtype == -1 ){
				play_se("../data/shortage.wav");
				return;
			}
			if( g_system->money < unit_type[newtype].cost ){
				play_se("../data/shortage.wav");
				return;
			}

			g_system->money -= unit_type[newtype].cost;
			system_draw(g_system->life, g_system->money, FALSE, 1);

			unit_remove( (unitSprite *)g_list_nth_data(unit_list, i) );
			unit_list = g_list_delete_link(unit_list, g_list_nth(unit_list, i));

			unit_new(area, pos_x+DIVIDE_SIZE/2, pos_y+DIVIDE_SIZE/2, newtype);

			play_se("../data/sell.wav");
			return;
		}
	}
}
